#ifndef _FUSION_CLUAVM_HPP_INCLUDED
#define _FUSION_CLUAVM_HPP_INCLUDED




#include "../fusCompileConfig.hpp"
#include "IScriptInterpreter.hpp"

#include <luabind/lua_include.hpp>
#include <luabind/luabind.hpp>
#include <string>

namespace fus {

//std::string const&,
typedef void (*TOnLuaErrorCallback)(std::string const&);

//
//typedef int(*pcall_callback_fun)(lua_State*);
//LUABIND_API void set_pcall_callback(pcall_callback_fun e);
//LUABIND_API pcall_callback_fun get_pcall_callback();

//extern TOnLuaErrorCallback onLuaErrorFunction;

// Utilities
int onLuaError(lua_State*);
int onLuaPanic(lua_State *s);
int onNewIndex(lua_State *s);
int onIndex(lua_State *L);
void stackDump (lua_State *L);




/** \brief TODO c
 * \todo check that string won't throw any exceptio ntoo
 * \param
 * \param
 * \return
 *
 */

class CLuaException : public std::exception
{
public:
    //!< Constructor
    CLuaException(
                  lua_State* L
                  );
                  //const std::string& message

    /// Destructor ( throw() shows destructor won't throw any exception )
    virtual ~CLuaException() throw();

    /// Return message associated to this exception
    virtual const char* what() const throw();

protected :
    std::string mMessage;       //!< Message associé à l'exception

};




/** \brief Set of helper functions
 *
 * \param
 * \param
 * \return
 *
 */

class CLuaVM :
    public IInterpreter
{

public:

    CLuaVM(lua_State* = 0);
    virtual ~CLuaVM();


    static lua_State* createVM();

    /** \brief Load libraries, update state to work with luabind
     *
     */
    static void loadVM( lua_State*);



    /** \brief Construct a message from stack when an error happens
     *
     * \param
     */
    static void setMessageHandler(lua_CFunction msgHandler ){
        MessageHandler = msgHandler;
    }


    static lua_CFunction getMessageHandler(){
        return MessageHandler ;
    }


    //static TOnLuaErrorCallback registerErrorHandlers(TOnLuaErrorCallback&);
//    static void registerErrorHandlers(TOnLuaErrorCallback&);



    /**
    Retreive string on top of stack
    **/
    bool retrieveStringFromStack(std::string&);

    void reset();

    /** \brief
     *
     * \param
     * \param
     * \return true if error
     *
     */
    //static bool checkReturnValue(const int&);
    static bool handleError(lua_State* state,std::string&);


    const std::basic_string<char> getErrorDescription()
    {
        return _errorMessage.str();
    };


    // TODO le sortir dehors ?
    void setErrorDescription(const int&);
    bool bind(const luabind::scope& scope);
//    bool RunCommand(const char* param);
    bool RunFile(const char* param) ;
    bool RunChunk(const int& array,const int& index) ;


//    bool loadFile(const char*) {
//        int temp;
//        return loadFile();
//    }

    /** \brief
     *
     * \param Filename
     * \param Mode Might be "b","t" or "bt" depending on if you want to load from binary or text
     * \return
     *
     */
    bool loadFile(const char*,const char* mode);
    bool loadString(const char*);


    // Weird ?
    bool loadChunkFromGlobals(const int&);

    int saveTopChunkIntoGlobals();

    void getGlobals( luabind::object& );
    lua_State* getVM() { return _vm;};
//    static bool execute(lua_State* L,const int& nbOfParameters,const int& nbOfResults);
    virtual bool execute(const int& nbOfParameters,const int& nbOfResults);

//    virtual bool executek( const int& nbOfParameters);

    // Prevents creation of new attributes to a class
    bool setSpecialNewIndex(const char*);

    int pushMessageHandler();
    void removeMessageHandler(const int&);
protected:
//    static TOnLuaErrorCallback onLuaErrorFunction;



    void destroyVM();

    static lua_CFunction MessageHandler;

    //int loadString(const char*);

    lua_State* _vm;
//    std::string _errorMessage;

    std::basic_ostringstream<char> _errorMessage;

};



}

#endif
