
function isTask(supposedTask)
end



local task_mt = {
	__newindex = forbidNewIndex,

	__tostring = function(task)
				--\nAction:\t
					local parentId = task["parent_task_id"] or "none";

					local blocking = "false";
					local blocked = "false";

					if task["blocking"] then
						  blocking = "true";
					end

					if task["blocked"] then
						  blocking = "true";
					end

					local subtask_ids = table.concat(task["subtask_ids"],"/");
					local str = "\nTask description:\nParent:\t"..parentId.."\nBlocking:\t"..blocking.."\nBlocked:\t"..blocked.."\nSubtask ids:\t"..subtask_ids.."\n";
					return str;
				end,

    -- retourner la description
	__concat   = function(op1,op2)
					local str =  tostring(op1) .. tostring(op2);
					return str;
				end,

	-- prevent
	__index     = forbidUnitializedIndex;
	__metatable = false
};



function Task.createNewTask()
    local task = {
					["action"] 			= false,				 --, not task or ,
					["launched"]  		= false,			-- to know if we need to resume coroutine with arguments
					["args"] 			= false,			-- arguments to pass when starting coroutine
					["parent_task_id"] 	= false,			-- if child task
					["subtask_ids"] 	= {},				-- children
					["blocking"] 		= false,			-- if blocks parent
					["blocked"] 		= false	,				-- each blocking child adds 1 when alive to this number (-1 when they die)
					["description"] 		= false					-- each blocking child adds 1 when alive to this number (-1 when they die)
				};

    setmetatable( task, task_mt);
    return task;
end

function Task.setup(task,fn,...)

    --assert( )

    --log_info("Creating new Task with id ["..id.."]");
	task["action"] 			= coroutine.create(fn) or error("Could not create coroutine");
	task["launched"]  		= false;			-- to know if we need to resume coroutine with arguments
	--table.pack ?
	task["args"] 			= {...}			-- arguments to pass when starting coroutine
	task["subtask_ids"] 	= {};				-- children
	task["parent_id"] 	    = nil;				-- children
	task["blocking"] 		= false			-- if blocks parent
	task["blocked"] 		= false					-- each blocking child adds 1 when alive to this number (-1 when they die)
    task["description"]     = "No description";

end


function Task.setDescription(task,description)
    assert( type(description) == "string", "description should be a string") );

    task["description"] = description;
end


-- function
function Task.addChild(task,childId)
    table.insert(task["subtasks_ids"], childId);
end

-- TODO
--function Task.setParent(task, parentId)
--
--	assert(parentId, "parameter of type "..type(parentId).."\n"..debug.traceback() );
--
----	log_info("Setting parent of task ["..childId.."] : new parent ["..parentId.."]")
--
--	local task = self.tasks[childId];
--	task["parent_task_id"] = parentId;
--
--	table.insert(self.tasks[parentId]["subtask_ids"], childId );
--end




function Task.unblock(task)

    task["blocked"] = false;
end




-- returns true when dead

-----------------------------------------------------------
--- Should return a value in TaskManager.ReturnCodes
-----------------------------------------------------------
function Task.process(task)

    local co = task["action"]

    local coStatus = coroutine.status(co);




    -- if first launch of coroutine
    --print("Treating id ["..id.."]" );
    if not task["launched"] then

--        log_info("Starting coroutine ["..id.."]");
--        self:resume_coroutine(id,co, table.unpack(task["args"]) );
        errorfree , arg1 = coroutine.resume( co, table.unpack(task["args"] )

        if errorfree then task["launched"] = true; end
        -- TODO
        --task["args"] = nil

        --print("launched:",task["launched"] );
    -- if dead coroutine
    elseif coStatus == "dead" then

        return TaskManager.ReturnCodes.DeadCoroutine;

    -- resume coroutine
    else
        errorfree, arg1  = coroutine.resume( co)
    end



    -- if we 're here, then we must have tried to resume or start coroutine, so we are checking
    -- check for errors
    if not errorfree then
        error("Could not start coroutine !\nError [",arg1,"]\nArguments: ",debug.traceback(co));
    end


    -- if arg1, then it has to be a task
    if arg1 then
        return TaskManager.ReturnCodes.NewTask , arg1
    end

end
