#include "IDialogState.hpp"
#include <cmath>
#include <irrlicht.h>
#include "../config.hpp"
#include "../game_engine.hpp"
//#include "../IGameConfig.hpp"
#include "../IdList.hpp"
#include <fusion/scripting/CLuaThread.hpp>
#include <fusion/scripting/CLuaVM.hpp>
#include <fusion/maths/macros.hpp>
#include <fusion/core/string.hpp>



using namespace irr;
using namespace fus;


IDialogState::IDialogState(const std::string& name) :
    IGameState( name,true,true,true,true,true )
    , _dialogTab(0)
    , _answers(0)
    , _text(0)
    , SpeakerImg(0)
    , SpeakerName(0)
    , WaitingForUserInput(false)
//    , SpeakerTexture(0)
{
    // cstor

}


IDialogState::~IDialogState()
{
    //
}


irr::s32 IDialogState::addAnswer(const std::string& label,const irr::s32& id)
{
    // TODO if an id is passed, use inertItem instead of addItem

    std::wstring wlabel;
    convertIntoWideString(label,wlabel);
    return _answers->addItem( wlabel.c_str() );
}



void IDialogState::clearAnswers()
{
    _answers->clear();
}



//! about speaker
void
IDialogState::setSpeakerName(const std::string& name)
{
//!


    std::wstring wname;
    convertIntoWideString(name,wname);
    _LOG_WARNING << "setSpakerName not implemented";
//    SpeakerName->setText( wname.c_str() );
}


void
IDialogState::setSpeakerTexture( const char* texturePath)
{
//!
//gui::IGUIImage* img = static_cast<gui::IGUIImage*>( _dialogTab->getElementFromId( NGuiIds::DialogImage ) ) ;
//    img->setImage ( speaker->getTexture() );
    video::ITexture* texture = driver()->getTexture(texturePath);
    if( ! texture )
    {
        _LOG_ERROR << "Could not load texture [" << texturePath << "]";
    }

    SpeakerImg->setImage( texture );
}


void
IDialogState::setMessage(const std::string& msg)
{

    std::wstring wmsg;
    convertIntoWideString(msg,wmsg);

    _INFO << "setting message: " << msg;
    // TODO is registered ?
//    gui::IGUIStaticText* text = static_cast<gui::IGUIStaticText*>( _dialogTab->getElementFromId( NGuiIds::DialogText ) ) ;


    _text->setText( wmsg.c_str() );
}


bool IDialogState::OnEvent(const irr::SEvent&)
{
    //!

    return false;
}


bool
IDialogState::load( CGameEngine* engine,irr::scene::ISceneManager* smgr,irr::video::ITexture* screenshot )
{
    //!
    if (IGameState::load( engine,smgr,screenshot ) == false)  return false;

    // Create gui
    GUI_CreateDialogTab();

    MainTab = _dialogTab;
    return true;
}


void
IDialogState::unload()
{
    //!
}


void
IDialogState::Draw()
{
    // We could update the screen only once in a while
    driver()->setRenderTarget(video::ERT_FRAME_BUFFER,true,true);

    //! Draw answers only when needed
    _answers->setVisible( (_answers->getItemCount () != 0) );

    MainTab->draw();
}


void
IDialogState::onUpdate(const TTimeUnit& elapsedTime)
{
    // TODO deplacer dans la
    CBindCollectionPlayer::TCache cachedInput;

    // TODO calculer la taille des reponses à partir de leur nombre
    //_answers->getItemCount ();
    //_dialog.second->_answers.size()

//    _answers->setSelected( _answers->getSelected() + 1 );

    // If there are questions
    //if( _answers->getItemCount () > 0 ){


    const irr::s32 itemCount = static_cast<irr::s32>( _answers->getItemCount () );
    irr::s32 selectedId = _answers->getSelected();

            //_INFO << "Item COUNT > 0" ;
    if( itemCount > 0)
    {

            //if( _mappingSet.getState(NPlayerInput::MoveForward) ){
            // MoveForward
            if( engine().getCachedInput()[NPlayerInput::ModeDown].pressed() )
            {

//                --selectedId;
//                if(selectedId < 0)
//                {
//                    selectedId = itemCount-1;
//                }
                selectedId = std::max( 0, selectedId - 1);
            }
            //else if( _mappingSet.getState(NPlayerInput::MoveBackward) ){
            else if( engine().getCachedInput()[NPlayerInput::ModeDown].pressed() )
            {

                //_answers->setSelected( _answers->getSelected() - 1 );
//                ++selectedId;
//                if( selectedId >= itemCount ){
//                    selectedId = 0;
//                }
                selectedId = std::min(itemCount, selectedId + 1);
            }

//        irr::core::clamp_
        _answers->setSelected( selectedId );
    }


    // IF shall resume script

//    TODO reactiver
    if( engine().getCachedInput ( )[NPlayerInput::MainShoot ].justPressed () )
    {

        int lastPlayerReply = _answers->getSelected();

        _INFO << "User replied id [" << lastPlayerReply << "]";

        WaitingForUserInput = false;
        onAnswer( lastPlayerReply );
    }
    else
    {
        //_INFO << "No reply yet";
    }
}


void
IDialogState::waitForUserInput()
{
    WaitingForUserInput = true;
}



void
IDialogState::GUI_CreateDialogTab()
{

    /// CONFIG
    const int gap = 10;
    const int totalHeight = 400;
    const int textHeight = 200;
    const video::SColor textColor(255,255,255,255);
    const video::SColor backgroundColor(100,0,0,0);
    //const int gap = 10;


    irr::gui::IGUIEnvironment* gui = engine().gui();

//
    // position2di
    //ENGINE->driver()->getViewPort().getWidth()
    //ENGINE->getScreenSize().Width
    core::recti comprehensiveRect(  0,0, engine().driver()->getViewPort().getWidth(), totalHeight );

    // "viewport in which we can draw"<<
    _INFO << "viewport in which we can draw" << comprehensiveRect;

    _dialogTab = gui->addTab ( comprehensiveRect );

    _dialogTab->setBackgroundColor( backgroundColor );
    _dialogTab->setID( NGuiId::DialogBox );
    _dialogTab->setDrawBackground(true);


    //_PORTRAIT_DIMENSION
    //irr::core::recti  textRect( core::position2di(_PORTRAIT_SIZE.Width + 10, 0 ), core::dimension2du(comprehensiveRect -,250) );

    irr::core::recti  textRect(
                        conf::_PORTRAIT_SIZE.Width + gap,
                        gap ,
                        comprehensiveRect.getWidth() - gap,
                        textHeight
                        );


    //decal(rect,10,10);
    gui::IGUIStaticText* text = gui->addStaticText(L"Default message",textRect,false);
    text->setID( NGuiId::DialogText );
    text->setOverrideColor ( textColor );
    _text = text;

    // TODO


    //text->setParent(  );
    _dialogTab->addChild(text);




    // speaker picture
    //core::dimension2di(100,50)
    irr::core::recti  srcRect( irr::core::position2di(gap,gap), conf::_PORTRAIT_SIZE );
    //irr::core::recti  textRect( _PORTRAIT_SIZE.Width + gap, 250 + gap , comprehensiveRect.getWidth() - gap , totalHeight - textHeight );

    SpeakerImg = gui->addImage( srcRect );
    SpeakerImg->setID( NGuiId::DialogImage );
    SpeakerImg->setText(L"Default image");
    SpeakerImg->setUseAlphaChannel (true);


    _dialogTab->addChild( SpeakerImg );
    // contains answers
    //gui->addListBox();

    //////////////////////////////////////////////////////////
    ////    Answers list
    //////////////////////////////////////////////////////////

    //irr::core::recti  answersRect( irr::core::position2di(_PORTRAIT_SIZE.Width, 270), core::dimension2di(300,100) );
    //- textHeight
    irr::core::recti  answersRect(
                        conf::_PORTRAIT_SIZE.Width + gap,
                        textHeight + gap ,
                        comprehensiveRect.getWidth() - gap ,
                        totalHeight
                        );

    gui::IGUIListBox* answers = gui->addListBox( answersRect );
    //answers->setOverrideColor ( video::SColor(255,255,255,255) );
    answers->addItem(L"Default answer");
    answers->setDrawBackground(true);
    answers->setID( NGuiId::DialogAnswers );
//    answers->setItemOverrideColor ( 0, EGUI_LBC_TEXT , textColor);
    _dialogTab->addChild( answers );


    _answers = answers;


    // TODO SpeakerName
}




// TODO checker si le joueur est mort


